#! /usr/bin/env python3

from typing import Dict, Any, Tuple
from configparser import ConfigParser
import os, json, logging, logging.config

from bottle import run, route, request

servicePath = os.path.dirname(os.path.abspath(__file__))

def loadConfig() -> Tuple[str, int, str, int]:
    """
    Получить puppetIP, puppetPort и certificate из файла config.ini
    :return: 4 значения: 1 - puppetIP, 2 - puppetPort, 3 - certificate, 4 - код ответа
    """
    conf = ConfigParser()
    conf.read(os.path.join(servicePath, 'config/config.ini'))
    try:
        puppetIP = conf.get('main', 'puppetIP')
    except:
        puppetIP = '192.169.10.71'
    try:
        puppetPort = conf.getint('main', 'puppetPort')
    except:
        puppetPort = 8140
    try:
        certificate = conf.get('main', 'certificate')
    except:
        certificate = '111'
    try:
        answerCode = conf.getint('main', 'answerCode')
    except:
        answerCode = 0
    return puppetIP, puppetPort, certificate, answerCode

def getAnswer(force = False, ansCode = -1) -> Dict[str, Any]:
    puppetIP, puppetPort, certificate, code = loadConfig()
    if ansCode != -1:
        code = ansCode
    if code == 10 and force:
        code = 0
    answers = {
        0: {
            'code': 0,
            'message': 'Ok',
            'initializeInfo': {
                'puppetServerIp': puppetIP,
                'puppetServerPort': puppetPort,
                'cashId': certificate
            }
        },
        1: {
            'code': 1,
            'message': 'Bad Cluster ID'
        },
        2: {
            'code': 2,
            'message': 'Shop Not Found'
        },
        3: {
            'code': 3,
            'message': 'Several Shop Found'
        },
        4: {
            'code': 4,
            'message': 'Cash Not Found'
        },
        5: {
            'code': 5,
            'message': 'Cash IP Not Match'
        },
        10: {
            'code': 10,
            'message': 'HardwareId Not Match'
        }
    }
    try:
        answer = answers[code]
    except KeyError:
        answer = answers[0]
    return answer

@route('/CSrest/rest/v2/cashes/initialization', method='POST')
def returnPuppetConf() -> str:
    logging.info(u'Получение запроса от кассы')
    body = json.loads(request.body.read())
    logging.info(u'Тело запроса:\n' + str(body))
    answer = json.dumps(getAnswer(u'force' in request.params and request.params[u'force'] == u'true' if True else False))
    logging.info(u'Отправка ответа')
    logging.info(u'Тело ответа:\n' + str(answer))
    return answer

def main() -> None:
    logging.basicConfig(format = u'[%(asctime)s] %(levelname)s - %(message)s', level = logging.INFO,
            filename = u'/linuxcash/logs/current/controlcentertest.log')
    logging.info(u'Старт сервиса')
    conf = ConfigParser()
    conf.read(os.path.join(servicePath, 'conf/config.ini'))
    try:
        host = conf.get('main', 'host')
    except:
        host = '0.0.0.0'
    try:
        port = conf.getint('main', 'port')
    except:
        port = 5000
    try:
        logging.info(u'Стартуем на %s:%s' % (host, port))
        run(host=host, port=port)
    except Exception as e:
        logging.error(e)

if __name__ == '__main__':
    main()
